"""
Cost catalog content
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Atiya atiya.atiya@mail.concordia.ca
Code contributors: Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""


class Content:
  """
  Content class
  """
  def __init__(self, archetypes):
    self._archetypes = archetypes

  @property
  def archetypes(self):
    """
    All archetypes in the catalog
    """
    return self._archetypes



"""
Energy System catalog emission system
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2023 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""


class EmissionSystem:
  """
  Emission system class
  """
  def __init__(self, system_id, name, system_type, parasitic_energy_consumption):

    self._system_id = system_id
    self._name = name
    self._type = system_type
    self._parasitic_energy_consumption = parasitic_energy_consumption

  @property
  def id(self):
    """
    Get system id
    :return: float
    """
    return self._system_id

  @id.setter
  def id(self, value):
    """
    Set system id
    :param value: float
    """
    self._system_id = value

  @property
  def name(self):
    """
    Get name
    :return: string
    """
    return self._name

  @name.setter
  def name(self, value):
    """
    Set name
    :param value: string
    """
    self._name = value

  @property
  def type(self):
    """
    Get type
    :return: string
    """
    return self._type

  @property
  def parasitic_energy_consumption(self):
    """
    Get parasitic_energy_consumption in ratio (Wh/Wh)
    :return: float
    """
    return self._parasitic_energy_consumption



"""
Energy System catalog distribution system
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2023 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""


class DistributionSystem:
  """
  Distribution system class
  """
  def __init__(self, system_id, name, system_type, supply_temperature, distribution_consumption_fix_flow,
               distribution_consumption_variable_flow, heat_losses):

    self._system_id = system_id
    self._name = name
    self._type = system_type
    self._supply_temperature = supply_temperature
    self._distribution_consumption_fix_flow = distribution_consumption_fix_flow
    self._distribution_consumption_variable_flow = distribution_consumption_variable_flow
    self._heat_losses = heat_losses

  @property
  def id(self):
    """
    Get system id
    :return: float
    """
    return self._system_id

  @id.setter
  def id(self, value):
    """
    Set system id
    :param value: float
    """
    self._system_id = value

  @property
  def name(self):
    """
    Get name
    :return: string
    """
    return self._name

  @name.setter
  def name(self, value):
    """
    Set name
    :param value: string
    """
    self._name = value

  @property
  def type(self):
    """
    Get type from [air, water, refrigerant]
    :return: string
    """
    return self._type

  @property
  def supply_temperature(self):
    """
    Get supply_temperature in degree Celsius
    :return: float
    """
    return self._supply_temperature

  @property
  def distribution_consumption_fix_flow(self):
    """
    Get distribution_consumption if the pump or fan work at fix mass or volume flow in ratio over peak power (W/W)
    :return: float
    """
    return self._distribution_consumption_fix_flow

  @property
  def distribution_consumption_variable_flow(self):
    """
    Get distribution_consumption if the pump or fan work at variable mass or volume flow in ratio
    over energy produced (Wh/Wh)
    :return: float
    """
    return self._distribution_consumption_variable_flow

  @property
  def heat_losses(self):
    """
    Get heat_losses in ratio over energy produced
    :return: float
    """
    return self._heat_losses



"""
Construction helper
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from hub.helpers import constants as cte


class ConstructionHelper:
  """
  Construction helper
  """
  # NREL
  _nrel_standards = {
    'ASHRAE Std189': 1,
    'ASHRAE 90.1_2004': 2
  }
  _reference_city_to_nrel_climate_zone = {
    'Miami': 'ASHRAE_2004:1A',
    'Houston': 'ASHRAE_2004:2A',
    'Phoenix': 'ASHRAE_2004:2B',
    'Atlanta': 'ASHRAE_2004:3A',
    'Los Angeles': 'ASHRAE_2004:3B',
    'Las Vegas': 'ASHRAE_2004:3B',
    'San Francisco': 'ASHRAE_2004:3C',
    'Baltimore': 'ASHRAE_2004:4A',
    'Albuquerque': 'ASHRAE_2004:4B',
    'Seattle': 'ASHRAE_2004:4C',
    'Chicago': 'ASHRAE_2004:5A',
    'Boulder': 'ASHRAE_2004:5B',
    'Minneapolis': 'ASHRAE_2004:6A',
    'Helena': 'ASHRAE_2004:6B',
    'Duluth': 'ASHRAE_2004:7A',
    'Fairbanks': 'ASHRAE_2004:8A'
  }
  nrel_window_types = [cte.WINDOW, cte.DOOR, cte.SKYLIGHT]

  nrel_construction_types = {
    cte.WALL: 'exterior wall',
    cte.INTERIOR_WALL: 'interior wall',
    cte.GROUND_WALL: 'ground wall',
    cte.GROUND: 'exterior slab',
    cte.ATTIC_FLOOR: 'attic floor',
    cte.INTERIOR_SLAB: 'interior slab',
    cte.ROOF: 'roof'
  }

  _reference_city_to_nrcan_climate_zone = {
    'Montreal': '6'
  }

  @staticmethod
  def yoc_to_nrel_standard(year_of_construction):
    """
    Year of construction to NREL standard
    :param year_of_construction: int
    :return: str
    """
    if int(year_of_construction) < 2009:
      standard = 'ASHRAE 90.1_2004'
    else:
      standard = 'ASHRAE 189.1_2009'
    return standard

  @staticmethod
  def city_to_reference_city(city):
    """
    City name to reference city
    :param city: str
    :return: str
    """
    # todo: Dummy function that needs to be implemented
    reference_city = 'Montreal'
    if city is not None:
      reference_city = 'Montreal'
    return reference_city

  @staticmethod
  def city_to_nrel_climate_zone(city):
    """
    City name to NREL climate zone
    :param city: str
    :return: str
    """
    reference_city = ConstructionHelper.city_to_reference_city(city)
    if reference_city not in ConstructionHelper._reference_city_to_nrel_climate_zone:
      reference_city = 'Baltimore'
    return ConstructionHelper._reference_city_to_nrel_climate_zone[reference_city]

  @staticmethod
  def city_to_nrcan_climate_zone(city):
    """
    City name to NRCAN climate zone
    :param city: str
    :return: str
    """
    reference_city = ConstructionHelper.city_to_reference_city(city)
    return ConstructionHelper._reference_city_to_nrcan_climate_zone[reference_city]
